﻿//  ID Text Unstyler, v1.0.0
//  June 22, 2018
//
//  Copyright 2018, Premedia Systems, Inc.
//
//  Written for: The Creative Pro Conference, 2018.
//
//  Written by (and report bugs to):
//      Jim Birkenseer
//      Premedia Systems, Inc.
//      jim@premediasystems.com
//

// CONSTANTS
var kScriptName = "ID Text Unstyler";
var kVersion = "1.0.0";

// Macintosh/Windows Compatibility Constants
if ( $.os.substr( 0, 3 ) == "Win" )
{
    var LINE_END_TEXT       = "\n\r"; // Windows
    var ALERT_NEW_PARA_TEXT = "\r\r"; // Windows
}
else // Assume Macintosh
{
    var LINE_END_TEXT       = "\r"; // Macintosh
    var ALERT_NEW_PARA_TEXT = "\r"; // Macintosh
}


// GLOBALS
var gSuccess = false;
var gProgressWindow = null;
var gProgressBarIncrement = 1;
var gProcessedParagraphCount = 0;
var gCurProgressBarValue = 0;

function RemoveTextStyling()
{
    var myResultStr = "";
    
    do
    {
        if (app.documents.length == 0)
        {
            gSuccess = true;
            myResultStr = kScriptName + " requires that a document be open.";
            break;
        }
        
        var myDoc = app.activeDocument;

        if (myDoc.stories.length == 0)
        {
            gSuccess = true;
            myResultStr = kScriptName + " did not modify your document." + ALERT_NEW_PARA_TEXT + "Details: The document doesn't contain any text to process.";
            break;
        }
        
        if (!confirm("Do you want " + kScriptName + " to unstyle all of the text in your document?" + ALERT_NEW_PARA_TEXT + "WHAT WILL HAPPEN: All of the text in your document will retain its formatting, but any paragraph or character style assignments made to the text will be removed. All existing paragraph and character styles will remain without any modifications.\r"))
        {
            gSuccess = true;
            myResultStr = kScriptName + " did not modify your document." + ALERT_NEW_PARA_TEXT + "Details: User chose not to process the document."; 
            //myResultStr = ""; 
            break;
        }
    
        
        try
        {
            var totalParagraphs = 0;
            for (var k = 0; k < myDoc.stories.length; ++k)
            {
                totalParagraphs += myDoc.stories[k].paragraphs.length;            
            }
    
            gProgressBarIncrement = Math.min( (Math.ceil(totalParagraphs/100)), 20);
        
            var totalPBSteps = Math.ceil(totalParagraphs / gProgressBarIncrement);

            PBBuildProgressWindow(totalPBSteps);
            gProgressWindow.show(); // Shows, but is hidden
       
            for (var storyIndex = 0; storyIndex < myDoc.stories.length; ++storyIndex)
            {
                var curStory = myDoc.stories[storyIndex];
                //$.writeln("Story: " + storyIndex);
        
                // Process the paragraphs in the table cells of the current story.
                for (var tableIndex = 0; tableIndex < curStory.tables.length; ++tableIndex)
                {
                    var curTable = curStory.tables[tableIndex];
            
                    for (var cellIndex = 0; cellIndex < curTable.cells.length; ++cellIndex)
                    {           
                        var curCell = curTable.cells[cellIndex];
                
                        for (var textRangeIndex = 0; textRangeIndex < curCell.textStyleRanges.length; ++textRangeIndex)
                        {
                            SetNoneCharacterStyle(curCell.textStyleRanges[textRangeIndex]);
                        }
            
                        for (var paraIndex = 0; paraIndex < curCell.paragraphs.length; ++paraIndex)
                        {
                            SetNoParagraphStyle(curCell.paragraphs[paraIndex]);
                        }
                    }
                }
        
                // Process footnotes attached to the story.
                for (var footnoteIndex = 0; footnoteIndex < curStory.footnotes.length; ++footnoteIndex)
                {
                    var curFootnote = curStory.footnotes[footnoteIndex];

                    for (var textRangeIndex = 0; textRangeIndex < curFootnote.textStyleRanges.length; ++textRangeIndex)
                    {
                        SetNoneCharacterStyle(curFootnote.textStyleRanges[textRangeIndex]);
                    }
    
                    for (var paraIndex = 0; paraIndex < curFootnote.paragraphs.length; ++paraIndex)
                    {
                        SetNoParagraphStyle(curFootnote.paragraphs[paraIndex]);
                    }
                }
   
                // Process the text ranges in the paragraphs in curent story.
                for (var textRangeIndex = 0; textRangeIndex < curStory.textStyleRanges.length; ++textRangeIndex)
                {
                    SetNoneCharacterStyle(curStory.textStyleRanges[textRangeIndex]);
                }
           
                // Process the paragraphs in the curent story.
                for (var paraIndex = 0; paraIndex < curStory.paragraphs.length; ++paraIndex)
                {
                    SetNoParagraphStyle(curStory.paragraphs[paraIndex]);
                    ++gProcessedParagraphCount;
                    
                    if (gProcessedParagraphCount % gProgressBarIncrement == 0)
                    {
                        ++gCurProgressBarValue;
                        
                        if ( (gProgressWindow == null) /*&& ( (new Date()).getTime() - gStartTime ) > gProgressShowDelayMSec */)
                        {
                            // Create the progress bar window
                            PBBuildProgressWindow(totalPBSteps);
                            gProgressWindow.show(); // Shows, but is hidden
                            gProgressWindow.ProgressBar1.value = gCurProgressBarValue;
                        }
                        else if (gProgressWindow)
                        {
                            gProgressWindow.ProgressBar1.value = gCurProgressBarValue;
                        }
                    }
                }
            }
        }
        catch (e)
        {
            myResultStr = kScriptName + " encountered an error before finishing. Choose \"Undo " + kScriptName + "\" from InDesign's Edit menu to undo any partial changes made to your document." + ALERT_NEW_PARA_TEXT + "Details: \"" + e + "\"";
            break;
        }

        gSuccess = true;
        myResultStr = kScriptName + " completed successfully." + ALERT_NEW_PARA_TEXT + "WHAT HAPPENED: The text is now formatted without using any paragraph or character styles. No paragraph or characters styles have been deleted from the document.";
    
    } while (false);
    
    if (gProgressWindow != null)
    {   
        gProgressWindow.close();
        gProgressWindow = null;
    }
    
    alert(myResultStr + "\r\rPlease visit premediasystems.com and use our contact form to provide us with your feedback about " + kScriptName + ", v" + kVersion + ".");
}

function SetNoneCharacterStyle(theTextStyleRun)
{
    theTextStyleRun.applyCharacterStyle(app.activeDocument.characterStyles.itemByName("[None]"));    
}

function SetNoParagraphStyle(theParagraph)
{
    theParagraph.applyParagraphStyle(app.activeDocument.paragraphStyles.itemByName("[No Paragraph Style]"), false);
}

//================== PROGRESS BAR WINDOW CODE START ===================\\

function PBAdvanceProgressBar()
{
	// Don't interact with a non-existant progress window!
	if (gProgressWindow == null) return;
	
	// For best visual appearance, please set myIncrement to a value that is a factor of 100.
	var myIncrement = 1;
	gProgressWindow.ProgressBar1.value += myIncrement;
}

function PBBuildProgressWindow(numSteps)
{
	gProgressWindow = new Window('palette', kScriptName, undefined);
	gProgressWindow.margins= [25,25,25,25];
	gProgressWindow.visible = false;
	
	gProgressWindow.StaticText1 = gProgressWindow.add("statictext", undefined, "Processing Stories...");		
	gProgressWindow.StaticText1.alignment = "Left";
	gProgressWindow.ProgressBar1 = gProgressWindow.add("progressbar",[12, 12, 180, 24],0,numSteps);
	gProgressWindow.ProgressBar1.size= [280,14];
	gProgressWindow.ProgressBar1.enabled = true;
	gProgressWindow.ProgressBar1.active = true;
	gProgressWindow.ProgressBar1.visible = true;
}

function PBCloseProgressWindow()
{
	if( gProgressWindow != null )
	{
		gProgressWindow.close();
		gProgressWindow = null;
	}
}

function PBShowProgressWindow()
{
	if( gProgressWindow != null )
	{
		gProgressWindow.show();
		gProgressWindow.active = true;
	}
}

//================== PROGRESS BAR WINDOW CODE END ===================\\

function main()
{
    RemoveTextStyling();
}

// Run with single undo if possible.
var myMajorVersion = parseInt((app.version.split("."))[0]);
if (myMajorVersion >= 6) // CS4 or newer
{
    // When run in ID CS4, and newer, a single undo will be available. Older ID versions
    //  will likely work, but there will be no single undo option.
    app.doScript(main, ScriptLanguage.javascript, undefined, UndoModes.entireScript, kScriptName);       
}
else
{
    main();
}
